local grafana = import 'grafonnet/grafana.libsonnet';
local barGaugePanel = grafana.barGaugePanel;
local dashboard = grafana.dashboard;
local row = grafana.row;
local template = grafana.template;
local logPanel = grafana.logPanel;
local prometheus = grafana.prometheus;
local tablePanel = grafana.tablePanel;
local tablePanel = grafana.tablePanel;
local singleStat = grafana.singlestat;
local gaugePanel = grafana.gaugePanel;
local graphPanel = grafana.graphPanel;
local template_cmn = import '../common/datasource_common.libsonnet';

local steps = [ 
     {color: 'green', value: 0,  op: 'gt'},
     {color: 'orange', value: 52428800, op: 'gt'},
     {color: 'red', value: 419430400, op: 'gt'},
     ];

local steps_1 = [ 
     {color: 'green', value: 0,  op: 'gt'},
     {color: 'orange', value: 21474836480, op: 'gt'},
     {color: 'red', value: 42949672960, op: 'gt'},
     ];



local steps_2 = [ 
     {color: 'green', value: 10, text: 'High'},
     {color: 'green', value: 20, text: 'Low', },
     ];

grafana.dashboard.new(
  'SpringBoot App Dashboard',
  uid='ABCJVM123',
  refresh='1m',
  time_from='now-6h',
  editable=true,
  tags=['java']
)
.addTemplate(
  template_cmn.dataSourceTemplate
  )
.addTemplate(
  template.new(
    'instance',
    'Prometheus',
    'label_values(jvm_threads_daemon_threads, instance)',
    label='instance',
    refresh='time',
  )
)
.addTemplate(
  template.new(
    'job',
    'Prometheus',
    'label_values(jvm_threads_daemon_threads{instance="$instance"}, job)',
    label='Job',
    refresh='time',
  )
).addTemplate(
  template.new(
    'env',
    'Prometheus',
    'label_values(jvm_threads_current, env)',
    label='Environment',
    refresh='time',
  )
).addRow(
  row.new(
    'Performance Stats',  
  )
  .addPanel(
    graphPanel.new(
      'CPU Usage',
      span=6,
      fill=1,
      min=0,
      format='',
      decimals=2,
      legend_values=true,
      legend_min=true,
      legend_max=true,
      legend_current=true,
      legend_total=false,
      legend_avg=true,
      legend_alignAsTable=true,
    )
    .addTarget(
      prometheus.target(
        'system_cpu_usage',
        legendFormat='CPU Usage',
        datasource='Prometheus',
      )
    ),
    gridPos = { h: 5, w: 30, x: 0, y: 0}
    ).addPanel(
    graphPanel.new(
      'System Load',
      span=6,
      fill=1,
      min=0,
      format='',
      decimals=2,
      legend_values=true,
      legend_min=true,
      legend_max=true,
      legend_current=true,
      legend_total=false,
      legend_avg=true,
      legend_alignAsTable=true,
    )
    .addTarget(
      prometheus.target(
        'system_load_average_1m',
        legendFormat='System Load',
        datasource='Prometheus',
      )
    ),
    gridPos = { h: 5, w: 30, x: 0, y: 0}
    )
    ).addRow(
     row.new(
    'Thread Information',
    ).addPanel(
        grafana.statPanel.new('Stat Panel representation of Jvm Threads state',
        datasource='Prometheus',
        allValues=false,
        colorMode='background',
        reducerFunction='lastNotNull',
        graphMode='none'
      ).addThresholds(steps_2)
      .addTarget(
        prometheus.target(
          hide=false,
          expr='jvm_threads_states_threads{job="$job",instance="$instance"}',
          legendFormat='{{state}}',
        )
      ),
       gridPos={ h: 5, w: 30, x: 0, y: 20 }
    ).addPanel(
        grafana.statPanel.new('JVM Memory Max Bytes',
        datasource='Prometheus',
        allValues=false,
        colorMode='background',
        graphMode='none',
        unit='bytes'
      ).addThresholds(steps_2)
      .addTarget(
        prometheus.target(
          hide=false,
          expr='jvm_memory_max_bytes{job="$job",instance="$instance"}',
          legendFormat='{{id}}',
        )
      ),
       gridPos={ h: 5, w: 30, x: 0, y: 20 }
    ).addPanel(
        grafana.statPanel.new('Server Requests Count',
        datasource='Prometheus',
        allValues=false,
        colorMode='background',
        graphMode='none',
        unit=''
      ).addThresholds(steps_2)
      .addTarget(
        prometheus.target(
          hide=false,
          expr='sum(http_server_requests_seconds_count) by (outcome)',
          legendFormat='{{outcome}}',
        )
      ),
       gridPos={ h: 5, w: 30, x: 0, y: 20 }
    )
    ).addRow(
  row.new(
    'Basic Stats',  
  )
  .addPanel(
    singleStat.new(
      'Application Up time',
      span=4,
      prefix='',
      format='s',
      postfix='',
      colorBackground=true,
      colorValue=false,
      thresholds="10,25",
      colors=[
      'rgba(237, 129, 40, 0.89)',
      '#00FF00',
      'green',
    ]
    ) 
    .addTarget(
      prometheus.target(
        'time()-process_start_time_seconds{env="$env",job="$job",instance="$instance"}',
        datasource='Prometheus',
      )
    )
  ).addPanel(gaugePanel.new('Jvm buffer capacity',
        datasource='Prometheus',
        min=0,
        max=1073741824,
        unit='bytes',
        showThresholdLabels=false,
        showThresholdMarkers=true,
        transparent=false,
    ).addThresholds(steps)
     .addTarget(
        prometheus.target(
          expr='max(jvm_buffer_total_capacity_bytes{job="$job",instance="$instance"})',
          datasource='Prometheus',
          legendFormat='buffer capacity'
        )
      ),
      gridPos={ h: 5, w: 30, x: 0, y: 0 }
    ).addPanel(gaugePanel.new('Jvm Memory Max Bytes',
        datasource='Prometheus',
        min=0,
        max=53687091200,
        unit='bytes',
        showThresholdLabels=false,
        showThresholdMarkers=true,
        transparent=false,
    ).addThresholds(steps_1)
     .addTarget(
        prometheus.target(
          expr='max(jvm_memory_max_bytes{job="$job",instance="$instance"})',
          datasource='Prometheus',
          legendFormat='buffer capacity'
        )
      ),
      gridPos={ h: 15, w: 30, x: 0, y: 0 }
    )
    )
     .addRow(
  row.new(
    'HikariCP Statistics',  
  )
  .addPanel(
    singleStat.new(
      'Connection Size',
      span=6,
      prefix='',
      postfix='',
      colorBackground=true,
      colorValue=false,
      thresholds="10,25",
      colors=[
      'rgba(237, 129, 40, 0.89)',
      '#00FF00',
      'green',
    ]
    ) 
    .addTarget(
      prometheus.target(
        'hikaricp_connections{env="$env",job="$job",instance="$instance"}',
        datasource='Prometheus',
      )
    )
  )
   .addPanel(
    singleStat.new(
      'Connection Timeout Count',
      span=6,
      prefix='',
      postfix='',
      colorBackground=true,
      colorValue=false,
      thresholds="10,25",
      colors=[
      'rgba(237, 129, 40, 0.89)',
      '#00FF00',
      'green',
    ]
    ) 
    .addTarget(
      prometheus.target(
        'hikaricp_connections_timeout_total{env="$env",job="$job",instance="$instance"}',
        datasource='Prometheus',
      )
    )
  )
  )  